/*==========================================================================*/
/*             |                                                            */
/* Filename    | TCPCL.JAVA                                                 */
/*             |                                                            */
/* Purpose     | Simple TCP client application.                             */
/*             |                                                            */
/* Remarks     | This client connects to the host and port specified on     */
/*             | the command line. It awaits an echo-server on the remote   */
/*             | end replying any user console input exept for 'exit'       */
/*             | which closes the connection and terminates the client.     */
/*             |                                                            */
/* Copyright   | Smart Network Devices GmbH                                 */
/*             | Johannes-Geller-Str. 13                                    */
/*             | 41462 Neuss                                                */
/*             | Germany                                                    */
/*             |                                                            */
/* Created     | January 5, 2000                                            */
/*             |                                                            */
/* Last Change | January 21, 2000 by Peter Duchemin                         */
/*             |                                                            */
/*==========================================================================*/

import java.io.*;
import java.net.*;

public class tcpcl
{
   static volatile boolean close_from_server = false;

   public static void main(String args[])
   {
      try {
         // Check the number of arguments
         if (args.length < 2) {
            throw new IllegalArgumentException("Wrong number of arguments");
         }
    
         // Parse the arguments
         final String host = args[0];
         int port = Integer.parseInt(args[1]);
      
         // Set up streams for reading from and writing to the console.
         // The 'to_user' stream is final for use in the anonymous class below.
         BufferedReader    from_user = new BufferedReader(new InputStreamReader(System.in));
         final PrintWriter to_user   = new PrintWriter(new OutputStreamWriter(System.out));

         // Connect to the specified host and port
         final Socket s = new Socket( host, port );
         to_user.println("Connected to server " + host);
         to_user.flush();

         // Set up streams for reading and writing to the server.
         // The 'from_server' is final for use in the anonymous class below.
         final BufferedReader from_server = new BufferedReader(new InputStreamReader(s.getInputStream()));
         PrintWriter          to_server   = new PrintWriter(new OutputStreamWriter(s.getOutputStream()));
         
         // Create a thread that gets output from the server and displays 
         // it to the user. We use a separate thread for this so that we can
         // receive server messages asynchronously from user's console input.
         Thread t = new Thread() 
         {
            public void run() {
               try {
                  while(true) 
                  {                     
                     // Create a receive buffer
                     String rxmsg;

                     // Wait to receive data from the server. This method will block, and 
                     // when it returns a packet has already been received in the buffer
                     // or the connection has been terminated by the peer.
                     if((rxmsg = from_server.readLine()) != null) { 
                        to_user.println(rxmsg);
                        to_user.flush(); 
                     } 
                     else {
                        // Termination by the remote server
                        to_user.println("Server " + host + " has disconnected.");
                        to_user.println("Type 'exit' to terminate local client.");
                        to_user.flush();
                        s.close(); 
                        close_from_server = true;
                        break;
                     }
                  }
               } 
               catch(Exception e) { }
            } 
         };
   
         // Priority settings
         t.setPriority(Thread.currentThread().getPriority() + 1);

         // Now start the server-to-user thread
         t.start(); 

         String line;

         // While receiving in a separate thread, read the user's input
         // and pass it to the server in this thread.
         while( ( line = from_user.readLine() ) != null ) {
            // Local termination of the client by typing 'exit'
            if( ( line.equals("EXIT") || line.equals("exit") ) && ( line.length() == 4 ) ) {
               if( !close_from_server ) {
                  to_user.println("Disconnecting from server " + host + " on user request.");
                  to_user.flush();
               }
               break;
            }

            // Send user's console input to the server
            to_server.println(line);
            to_server.flush(); 
         }

         if ( !close_from_server ) {
            // Close the socket, which will also interrupt the blocking readLine()
            // method in the receiving thread.
            s.close(); 
         }
      }

      // If anything goes wrong, print an error message
      catch(IllegalArgumentException e) {
         System.err.println(e);
         System.err.println("Usage: java tcpcl <hostname> <port>");
      }
      catch(Exception e) {
         System.err.println(e);
      }     
   }
}